;**********************************************************************
;* gboot - gboot_crtstart.a51
;* A simple USB HID bootloader for silicon laboratories USB mcus
;*
;* Copyright (c) 2010 Gabriele Gorla
;*
;* This program is free software: you can redistribute it and/or modify
;* it under the terms of the GNU General Public License as published by
;* the Free Software Foundation, either version 3 of the License, or
;* (at your option) any later version.
;*
;* This program is distributed in the hope that it will be useful,
;* but WITHOUT ANY WARRANTY; without even the implied warranty of
;* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;* GNU General Public License for more details.
;*
;* You should have received a copy of the GNU General Public License
;* along with this program.  If not, see <http://www.gnu.org/licenses/>.
;**********************************************************************
.module gboot_crtstart

.area HOME (CODE)
.area CSEG (CODE)

.globl _user_code_start
.globl __start__stack
.globl _gboot_main

.globl _TINY_STRINGDESCTABLE
.globl _String0Desc
.globl _String1Desc
.globl _String2Desc

.globl _GBOOT_INFO_PACKET
.globl _FLASH_INFO_PACKET
.globl _DEV_INFO_PACKET
.globl _EXT_INFO_PACKET

.globl _ZERO_PACKET
.globl _ONES_PACKET

.globl _gboot_version
.globl _user_flash_end
.globl _hw_pid
.globl _hw_rev

.globl _USB0ADR
.globl _USB0DAT
.globl _USB_POLL
.globl _USB_WRITE0
.globl _USB_WRITE1

;--------------------------------------------------------
; Stack segment in internal ram
;--------------------------------------------------------
.area SSEG (DATA)

__start__stack:
.ds 1

;--------------------------------------------------------
; reset vector and interrupt redirection
;--------------------------------------------------------
.area HOME (CODE)

gboot_interrupt_vect:
ljmp _gboot_main; reset vector
ajmp _user_code_start + 0x03 ; interrupt 0
.db #0xFF  ; unused
_GBOOT_INFO_PACKET:  ; offset 0x06
.byte (_gboot_version)     ; gboot minor version
.byte (_gboot_version>>8)  ; gboot api + gboot major version
.byte #0x20  ; silabs mcu 
.byte #0xF3  ; 
.db #0xFF  ; unused
ajmp _user_code_start + 0x0B; interrupt 1
.db #0xFF  ; unused
_FLASH_INFO_PACKET: ; offset 0x0e
.byte (_user_code_start>>8)   ; flash first page 
.byte (_user_flash_end>>8)    ; flash last page
.byte #0x09                   ; flash page size
.byte #0xff                   ; reserved
.db #0xFF  ; unused
ajmp _user_code_start + 0x13 ; interrupt 2
.db #0xFF  ; unused
_DEV_INFO_PACKET:  ; offset 0x16
.byte (_hw_pid)    ; USB PID 
.byte (_hw_pid>>8) ; USP_PID
.byte (_hw_rev)    ; hw revision 
.byte #0xFF        ; pcb rev
.db #0xFF  ; unused
ajmp _user_code_start + 0x1B ; interrupt 3
.db #0xFF  ; unused
_EXT_INFO_PACKET: ; offset 0x1e
.byte #0xff  ; reserved
.byte #0xff  ; reserved
.byte #0xff  ; reserved
.byte #0xff  ; reserved
.db #0xFF  ; unused
ajmp _user_code_start + 0x23; interrupt 4
.db #0xFF  ; unused
_DEV_SERIAL:  ; offset 0x26
.byte #0xff  ; device serial # in bcd
.byte #0xff  ; 
.byte #0xff  ; 
.byte #0xff  ; 
.db #0xFF  ; unused
ajmp _user_code_start + 0x2B ; interrupt 5
.db #0xFF  ; unused
_DEV_CONFIG: ; offset 0x2e
.byte #0xff  ; device specific
.byte #0xff  ; 
.byte #0xff  ; 
.byte #0xff  ; 
.db #0xFF  ; unused
ajmp _user_code_start + 0x33 ; interrupt 6
_TINY_STRINGDESCTABLE:
.byte _String0Desc,(_String0Desc >> 8)	
.byte _String1Desc,(_String1Desc >> 8)	
.byte _String2Desc,(_String2Desc >> 8)	
ajmp _user_code_start + 0x3B; interrupt 7
_String0Desc:
.db #0x04  ; Sring0Len
.db #0x03  ; DSC_STRING
.db #0x09  ;  
.db #0x04  ;
.db #0xFF  ; unused
.db #0xFF  ; unused
ajmp _user_code_start + 0x43 ; interrupt 8
_USB_POLL:
00101$:
	mov	a,_USB0ADR
	jb	acc.7,00101$
	ret
ajmp _user_code_start + 0x4B ; interrupt 9
_USB_WRITE0:
mov	_USB0DAT,#0x00
ret
_ZERO_PACKET:
.db #0x00  ; zeros packet
.db #0x00
ajmp _user_code_start + 0x53 ; interrupt 10
_USB_WRITE1:
mov	_USB0DAT,#0x01
ret
_ONES_PACKET:
.db #0x01  ; ones packet
.db #0x00
ajmp _user_code_start + 0x5B ; interrupt 11
.ds 6
ajmp _user_code_start + 0x63 ; interrupt 12
.ds 6
ajmp _user_code_start + 0x6B ; interrupt 13
.ds 6
ajmp _user_code_start + 0x73 ; interrupt 14
.ds 6
ajmp _user_code_start + 0x7B ; interrupt 15
