/**********************************************************************
 * gboot - gboot_usb_core.h
 * A simple USB HID bootloader for silicon laboratories USB mcus
 *
 * Copyright (c) 2010 Gabriele Gorla
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 **********************************************************************/

#include "gboot.h"
#include "gboot_usb_core.h"
#include "gboot_usb_descriptor.h"
#include "gboot_usb_interface.h"

#pragma codeseg USBSEG


extern __code device_descriptor_t DEVICEDESC;
extern __code config_descriptor_set_t HIDCONFIGDESC;
extern __code BYTE HIDREPORTDESC[];


#ifdef GBOOT_TINY
extern __code unsigned char* __code TINY_STRINGDESCTABLE [];
#define STRINGDESCTABLE TINY_STRINGDESCTABLE
#else
extern __code unsigned char* __code STD_STRINGDESCTABLE [];
#define STRINGDESCTABLE STD_STRINGDESCTABLE
#endif

void USB_WRITE0();
void USB_WRITE1();

extern setup_packet_t SETUP;             // Buffer for current device request


extern const __code unsigned char ONES_PACKET[2];
extern const __code unsigned char ZERO_PACKET[2];

unsigned char ControlReg;           // Temporary storage for EP control

//-----------------------------------------------------------------------------
// Global Variable Definitions
//-----------------------------------------------------------------------------
unsigned char USB0_STATE;              // Holds the current USB State

setup_packet_t SETUP;                  // Buffer for current device
                                       // request information
unsigned char DATASIZE;                 // Size of data to return

rx_dataptr_t RXDATAPTR;                // Pointer to received data
tx_dataptr_t TXDATAPTR;                // Pointer to data to send

unsigned char EP0_STATUS;           // Holds the status for each endpoint

__bit rt_ok;
__bit EP1_STATUS; // IDLE=0, HALT=1


//-----------------------------------------------------------------------------
// Definitions
//-----------------------------------------------------------------------------
// Redefine existing variable names to refer to the descriptors within the
// HID configuration descriptor.
#define ConfigDesc      (HIDCONFIGDESC.hid_configuration_descriptor)
#define HidDesc         (HIDCONFIGDESC.hid_descriptor)


#if 0
//#ifndef GBOOT_TINY
void USB_POLL()
{
  while(USB0ADR & 0x80);
}
#endif

/*unsigned char USB_READ_REG(unsigned char a)
{
  //USB_POLL();
  while(USB0ADR & 0x80); 
  USB0ADR = (0x80|a);
  USB_POLL(); 
  //  while(USB0ADR & 0x80); 
  return USB0DAT;
  }*/

void USB_WRITE_REG(unsigned char a, unsigned char d)
{
  USB_POLL();
  USB0ADR = (a); 
  USB0DAT = d;
}

void USB_WRITE_E0CSR_END(void)
{
  USB_POLL();
  USB0ADR = E0CSR;
  USB0DAT = (rbSOPRDY | rbDATAEND);
  rt_ok=1;
}
void USB_WRITE_E0CSR_SOPRDY(void)
{
  USB_POLL();
  USB0ADR = E0CSR;
  USB0DAT = (rbSOPRDY);
  rt_ok=1;
}

void USB_WRITE_E0CSR(unsigned char d)
{
  //  USB_POLL();
  while(USB0ADR & 0x80); 
  USB0ADR = E0CSR; 
  USB0DAT = d;
}

void USB_WRITE_INDEX0()
{
  USB_POLL();
  USB0ADR = INDEX; 
#ifdef GBOOT_TINY
  USB_WRITE0();
#else
  USB0DAT = 0;
#endif
}

void USB_WRITE_INDEX1()
{
  USB_POLL();
  USB0ADR = INDEX;
#ifdef GBOOT_TINY
  USB_WRITE1();
#else
  USB0DAT = 1;
#endif
}

/*void USB_READ_E0CSR()
{
  USB_POLL();
  //  while(USB0ADR & 0x80); 
  USB0ADR = (0x80|E0CSR);
  USB_POLL(); 
  //  while(USB0ADR & 0x80); 
  ControlReg = USB0DAT;
  }*/

/*
void Fifo_Read (unsigned char addr, unsigned char uNumBytes,
		unsigned char __data * pData) 
{
  unsigned char i;
  
  if (uNumBytes) {                     // Check if >0 bytes requested,
    USB0ADR = (addr);                // Set address
    USB0ADR |= 0xC0;                 // Set auto-read and initiate
    // first read
    
    // Unload <NumBytes> from the selected FIFO
    for(i=0;i< (uNumBytes);i++) {
      //USB_POLL();
      while (USB0ADR & 0x80);       // Wait for BUSY->'0' (data ready)
      pData[i] = USB0DAT;           // Copy data byte
    }
    
    //while(USB0ADR & 0x80);         // Wait for BUSY->'0' (data ready)
    USB0ADR = 0;                     // Clear auto-read
  }
}
*/
#ifndef GBOOT_TINY
#define SET_ZERO_PACKET() SET_PACKET(0)
#else
void SET_ZERO_PACKET()
{
  __asm
    mov	_TXDATAPTR, #(_ZERO_PACKET)
    mov	(_TXDATAPTR + 1), #( _ZERO_PACKET >> 8 )
    mov	_DATASIZE,#0x01
  __endasm;

  EP0_STATUS = EP_TX;    // Put endpoint into transmit mode
  USB_WRITE_E0CSR_SOPRDY();
}
#endif

void SET_PACKET(__bit c)
{
#ifndef GBOOT_TINY
  TXDATAPTR = (tx_dataptr_t)&ZERO_PACKET;
  if(c)   TXDATAPTR=(tx_dataptr_t)&ONES_PACKET;

  DATASIZE=1;
  EP0_STATUS = EP_TX;    // Put endpoint into transmit mode
  USB_WRITE_E0CSR_SOPRDY();
#else
  c; // avoid unused parameter warning
  __asm
    acall _SET_ZERO_PACKET
    jnb	  _SET_PACKET_PARM_1,00102$
    mov	  _TXDATAPTR, #(_ONES_PACKET)
00102$:
  __endasm;
#endif
}


__bit check_IDX0_LEN(unsigned char len)
{
#ifndef GBOOT_TINY
  if((SETUP.wIndex.i==0)&&(SETUP.wLength==len))
    return 1;
  return 0;
#else
  len; // avoid unused parameter warning
  __asm
    ;	if((SETUP.wIndex.i==0)&&(SETUP.wLength==len))
    mov	a,(_SETUP + 0x0004)
    orl	a,((_SETUP + 0x0004) + 1)
    orl	a,((_SETUP + 0x0006) + 1)
    clr	c
    jnz	00102$
    mov	a,dpl
    ;	mov	a,r2
    cjne	a,(_SETUP + 0x0006),00102$
    ;   return 1;
    setb c
00102$:
  __endasm;
#endif
}

__bit check_VAL0_IDX0_LEN(unsigned char l)
{
  if(SETUP.wValue.i==0) return check_IDX0_LEN(l);
  else return 0;
}


//-----------------------------------------------------------------------------
// Get_Status
//-----------------------------------------------------------------------------
void Get_Status (void)                // This routine returns a two byte
{                                     // status packet to the host  

  if ( (SETUP.wValue.i  &&
	                              // If non-zero return length or data
	                              // length not
	(SETUP.wLength != 2)) || SETUP.wIndex.c[MSB] )
                                      // Equal to 2 then send a stall
                                      // indicating invalid request
    return;


  switch(SETUP.bmRequestType) {        // Determine if recipient was device,
                                       // interface, or EP
  case IN_DEVICE:
    if (SETUP.wIndex.c[LSB]==0)
      SET_ZERO_PACKET();
    // Send 0x00, indicating bus power and no
    // remote wake-up supported
    break;

  case IN_INTERFACE:
    if ((USB0_STATE == DEV_CONFIGURED) &&
	SETUP.wIndex.c[LSB])
      SET_ZERO_PACKET();
    // Only valid if device is configured
    // and non-zero index
    // Status packet always returns 0x00
    break;
    
  case IN_ENDPOINT:
    if ((USB0_STATE == DEV_CONFIGURED) 
        && (SETUP.wIndex.c[LSB] == IN_EP1) )
      SET_PACKET(EP1_STATUS);
    break;
    
  default:
    break;
  }
}

//-----------------------------------------------------------------------------
// Clear_Feature
//-----------------------------------------------------------------------------
void Clear_Feature()                  // This routine can clear Halt Endpoint
{                                      // features on endpoint 1

  // device is configured
  if ( (USB0_STATE == DEV_CONFIGURED) && 
       // Verify that packet was directed at an endpoint
       (SETUP.bmRequestType == OUT_ENDPOINT)&&
       // The feature selected was HALT_ENDPOINT
       (SETUP.wValue.c[LSB] == ENDPOINT_HALT)  &&
       // And that the request was directed at EP 1 in
       ((SETUP.wIndex.c[LSB] == IN_EP1) ) ) {
    USB_WRITE_INDEX1(); // Clear feature endpoint 1 halt
    USB_WRITE_REG (EINCSR1, rbInCLRDT);
    //      EP_STATUS[1] = EP_IDLE;    // Set endpoint 1 status back to idle
    EP1_STATUS=0;    // Set endpoint 1 status back to idle
    
    USB_WRITE_INDEX0();         // Reset Index to 0
    USB_WRITE_E0CSR_END();
  }
}

//-----------------------------------------------------------------------------
// Set_Feature
//-----------------------------------------------------------------------------
void Set_Feature (void)                // This routine will set the EP Halt
{                                      // feature for endpoint 1

  // Make sure device is configured, SETUP data
  if ((USB0_STATE == DEV_CONFIGURED) &&
      (SETUP.bmRequestType == OUT_ENDPOINT)&&
      // endpoint feature is selected
      (SETUP.wValue.c[LSB] == ENDPOINT_HALT) &&
      (SETUP.wIndex.c[LSB] == IN_EP1) )
    {
      USB_WRITE_INDEX1(); // Set feature endpoint 1 halt
      USB_WRITE_REG (EINCSR1, rbInSDSTL);
      //      EP_STATUS[1] = EP_HALT;
      EP1_STATUS=1;
      
      USB_WRITE_INDEX0();
      USB_WRITE_E0CSR_END();
      // Indicate SETUP packet has been serviced
      //rt_ok=1;
    }
}

//-----------------------------------------------------------------------------
// Set_Address
//-----------------------------------------------------------------------------
void Set_Address (void)                // Set new function address
{
   // Request must be directed to device
   if ((SETUP.bmRequestType == OUT_DEVICE) &&
   // with index and length set to zero.
       check_IDX0_LEN(0) &&
//          SETUP.wIndex.i ==0 &&
//          SETUP.wLength ==0 &&
   SETUP.wValue.c[MSB] == 0 && (SETUP.wValue.c[LSB] & 0x80) == 0 )
   {
     EP0_STATUS = EP_ADDRESS;          // Set endpoint zero to update
     // address next status phase
     if (SETUP.wValue.c[LSB] != 0)
       {
	 USB0_STATE = DEV_ADDRESS;        // Indicate that device state is now
	 // address
       }
     else
       {
	 USB0_STATE = DEV_DEFAULT;        // If new address was 0x00, return
       }                                   // device to default state
     
     USB_WRITE_E0CSR_END();
     // Indicate SETUP packet has
     // been serviced
     //rt_ok=1;
   }
}

//-----------------------------------------------------------------------------
// Get_Descriptor
//-----------------------------------------------------------------------------
void Get_Descriptor (void)             // This routine sets the data pointer
{                                      // and size to correct descriptor and
                                       // sets the endpoint status to transmit

  switch(SETUP.wValue.c[MSB])         // Determine which type of descriptor
    {                                   // was requested, and set data ptr and
    case DSC_DEVICE:                 // size accordingly
      TXDATAPTR = (tx_dataptr_t) &DEVICEDESC;
      DATASIZE = DEVICEDESC.bLength;
      break;
      
    case DSC_CONFIG:
      TXDATAPTR = (tx_dataptr_t) &ConfigDesc;
      // must swap if using BIG ENDIAN compiler
      DATASIZE=ConfigDesc.wTotalLength;
      break;
      
    case DSC_STRING:
      //  NOTE: if strings are added to this project, the hard-coded
      // value of 2 will need to be increased
      // THIS DOES NOT HANDLE MULTIPLE LANGUAGES
      
      if (SETUP.wValue.c[LSB] > 2)   // If asking for string that's N/A
	  return;
      TXDATAPTR = (tx_dataptr_t) STRINGDESCTABLE[SETUP.wValue.c[LSB]];
      DATASIZE = *TXDATAPTR;
      break;
    case DSC_HID:                       // HID class descriptor
      TXDATAPTR = (tx_dataptr_t)&HidDesc;
      DATASIZE = HidDesc.bLength;
      break;
    case DSC_HID_REPORT:                // HID report descriptor
      TXDATAPTR = (tx_dataptr_t)&HIDREPORTDESC;
      DATASIZE = HID_REPORT_DESCRIPTOR_SIZE;
      break;
      
    default:
      return; // unsupported request
   }
  
       if ((SETUP.wLength < DATASIZE))
       {
         DATASIZE = SETUP.wLength;
    }
  
  
  USB_WRITE_E0CSR_SOPRDY(); // Service SETUP Packet
  EP0_STATUS = EP_TX;             // Put endpoint in transmit mode
  //  rt_ok=1;
}

//-----------------------------------------------------------------------------
// Get_Configuration
//-----------------------------------------------------------------------------
void Get_Configuration (void)          // This routine returns current
{                                      // configuration value

  // This request must be directed to the device
  // and value==0, index==0, len==1
  if ( (SETUP.bmRequestType == IN_DEVICE)    &&
       check_VAL0_IDX0_LEN(1) ) {

    SET_PACKET(USB0_STATE == DEV_CONFIGURED); 
    // If the device is configured return 1 else return 0
  }
}

//-----------------------------------------------------------------------------
// Set_Configuration
//-----------------------------------------------------------------------------
void Set_Configuration (void)          // This routine allows host to change
{                                      // current device configuration value

  // Device must be addressed before configured
  // and request recipient must be the device
  // the index and length words must be zero

  if ((USB0_STATE != DEV_DEFAULT) &&
      (SETUP.bmRequestType == OUT_DEVICE) &&
      check_IDX0_LEN(0) &&
      SETUP.wValue.c[MSB]  == 0 && (SETUP.wValue.c[LSB] < 2)) {
    // gboot only supports config = 0,1
    if (SETUP.wValue.c[LSB]) {
      // Any configuration request
      // results in configuration being set to 1
      USB0_STATE = DEV_CONFIGURED;
      
      USB_WRITE_INDEX1();   // Change index to endpoint 1
      // Set DIRSEL to indicate endpoint 1 is IN/OUT
      USB_WRITE_REG (EINCSR2, rbInSPLIT);
      USB_WRITE_INDEX0();   // Set index back to endpoint 0
      
      EP1_STATUS=0;       // Set endpoint status to idle (enabled)
    } else {
      USB0_STATE = DEV_ADDRESS;     // Unconfigures device by setting state
      EP1_STATUS=1;                 // to address, and changing endpoint 1
    }
    
    USB_WRITE_E0CSR_END();
  }
}

//-----------------------------------------------------------------------------
// Get_Interface
//-----------------------------------------------------------------------------
void Get_Interface (void)              // This routine returns 0x00, since
{                                      // only one interface is supported by
                                       // gboot

  // If device is configured
  // and  recipient is an interface
  // and val==0, index==0, lenght==1
  if ((USB0_STATE == DEV_CONFIGURED) &&
      (SETUP.bmRequestType == IN_INTERFACE) &&
      check_VAL0_IDX0_LEN(1) ) {
    
    SET_ZERO_PACKET();
  }
}

//-----------------------------------------------------------------------------
// Set_Interface
//-----------------------------------------------------------------------------
void Set_Interface(void)
{
  // Make sure request is directed at interface
  // and all other packet values are set to zero
  if ( (SETUP.bmRequestType == OUT_INTERFACE)  &&
       check_VAL0_IDX0_LEN(0) ) { 
    
    USB_WRITE_E0CSR_END(); 
  }
}


void Handle_Request(void)
{
  rt_ok=0;
  
  switch(SETUP.bmRequestType&RT_TYPE_MASK) {
  case RT_TYPE_STD:
    switch (SETUP.bRequest) {       // Call correct subroutine to handle
      // each kind of standard request
      case GET_STATUS:
	Get_Status();
	break;
      case CLEAR_FEATURE:
	Clear_Feature();
	break;
      case SET_FEATURE:
	Set_Feature();
	break;
      case SET_ADDRESS:
	Set_Address();
	break;
      case GET_DESCRIPTOR:
	Get_Descriptor();
	break;
      case GET_CONFIGURATION:
	Get_Configuration();
	break;
      case SET_CONFIGURATION:
	Set_Configuration();
	break;
      case GET_INTERFACE:
	Get_Interface();
	break;
      case SET_INTERFACE:
	Set_Interface();
	break;
      default:
      }
    break;
  case RT_TYPE_CLASS:
    switch (SETUP.bRequest) {
    case GET_REPORT:
      //#ifdef GBOOT_TINY
      //  TXDATAPTR =  (tx_dataptr_t)&INFO_PACKET;
      //TXDATAPTR =  (__code unsigned char *)READ_PTR;
      //#else
      //      TXDATAPTR =  (tx_dataptr_t)&IN_PACKET;
      //#endif
      // TXDATAPTR is updated by the read command 
      EP0_STATUS = EP_TX;            // Endpoint 0 in transmit mode
      break;
    case SET_REPORT:
      RXDATAPTR = (rx_dataptr_t)&OUT_PACKET;
      EP0_STATUS = EP_RX;            // Endpoint 0 in receive mode
      break;
    default:
      goto end;
    }
    DATASIZE = CMD_PACKET_SIZE;
    // FIXME: should it send stall if the report does not exist???
    USB_WRITE_E0CSR_SOPRDY();
    break;
    // case RT_TYPE_VENDOR:
    //   break;
  default:
  }

 end:
  if(!rt_ok) {
    USB_WRITE_E0CSR(rbSDSTL);   // Set the send stall bit
    EP0_STATUS = EP_STALL;      // Put the endpoint in stall status
  }
}
